﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.Linq;

namespace Falcon.SugarApi
{
    /// <summary>
    /// Object类型扩展方法
    /// </summary>
    public static class ObjectExtend
    {
        /// <summary>
        /// 对source进行浅表复制，并复制到target中
        /// </summary>
        /// <typeparam name="TSource">原对象类型</typeparam>
        /// <param name="source">原对象</param>
        /// <param name="target">目标对象</param>
        public static TSource CloneTo<TSource>(this TSource source, [NotNull] object target) where TSource : class {
            _ = source ?? throw new ArgumentNullException(nameof(source));
            _ = target ?? throw new ArgumentNullException(nameof(target));
            var all = from s in source.GetType().GetProperties()
                      join t in target.GetType().GetProperties() on s.Name equals t.Name
                      select new { s, t };
            foreach (var item in all) {
                item.t.SetValue(target, Convert.ChangeType(item.s.GetValue(source), item.s.PropertyType));
            }
            return source;
        }

        /// <summary>
        /// 从原对象中浅表复制属性值
        /// </summary>
        /// <typeparam name="Ttarget">目标对象类型</typeparam>
        /// <param name="target">目标对象</param>
        /// <param name="source">原对象</param>
        /// <returns>目标对象</returns>
        public static Ttarget CloneFrom<Ttarget>(this Ttarget target, object source) where Ttarget : class {
            source.CloneTo(target);
            return target;
        }
    }
}
